# -*- coding: utf-8 -*-
#
# gphys に関係しないテクニック的なスクリプトをまとめる
# 2015/08/07 v1 作成. add_slash を追加 
# 2015/08/25 makeanim.rb を移す
# 2015/09/09 ext_time を追加. v2
# 2015/10/22 荻原 hour_to_lon を作成 ver2
#                 lon_to_index を作成 ver2
#                 ext_time を更新. sec に対応. v3
#
# 2015/10/27 荻原 draw_ctl.rb から ディレクトリがある場合の所作をこちらに移す
# 2015/11/17 荻原 laglange_interpolating を追加. 
#                 laglange_interpolating_core を追加. 
# 2015/12/17 荻原 vfratio を追加
$LOAD_PATH.push(Dir.pwd)
require 'rubygems'
require 'rmagick'
require 'gphys_const'
require 'optparse'

include Magick

########################################################################################
def add_slash(dir)
  #ディレクトリを指定した時に最後に/を入れても入れなくても最後に / を追加する
  hantei = dir.slice(dir.length-1, dir.length-1)
  if hantei != "/" then
    dir << "/"
  end
  return dir
end


def make_anime(animefig, delay, animefile)
  #gifアニメ作成スクリプト 

  images = ImageList.new()
  animefig.each_with_index do |fig, index|
    images[index] = Image.read(fig)[0]
  end
  #  images.iterations = 1#0にすると無限回繰り返す
  images.iterations = 0
  images.each do |img|
    img.delay = delay
  end
  images.write(animefile)

end

def ext_time(iyrs, iyre, vir_sp_dir, sp_file, sp_vname, vir_dir, file, vname)
  # sp_for_Mars_T021_MGS.nc から指定された var (topog, albed, tiのどれか) 緯度, 経度の値は同じで時間に拡張したファイルを作成する.一度 time.nc を作成した場合は作成しない. 
  # ls_array は指定する Ls の配列. 指定したい Ls の数だけ配列に加えて返す 
  # 複数年またぐ場合はバグる可能性が高いので注意.
  sp_dir = vir_sp_dir.dup
  dir = vir_dir.dup
  sp_dir = add_slash(sp_dir)
  dir = add_slash(dir)
  var = GPhys::IO.open(sp_dir+sp_file+".nc",sp_vname)

  vir = GPhys::IO.open(dir+file+".nc",vname)
  daysOfYear = 0
  DaysOfMonth.each do |i|
    daysOfYear += i
  end

  if ls.coord("time").get_att('units') == 'hour'
    ts = (iyrs - 1) * daysOfYear * DataNumOfDay
    te = iyre * daysOfYear * DataNumOfDay
  elsif ls.coord("time").get_att('units') == 'sec'
    ts = (iyrs - 1) * daysOfYear * DataNumOfDay * HoursOfSecond
    te = iyre * daysOfYear * DataNumOfDay * HoursOfSecond
  end
  if (vir.coord("time")[-1].val[0]) < te then
    te = vir.coord("time")[-1].val[0]
  end

  if File.exist?("./tmp/"+sp_file+"_ext_time.nc")
    time = GPhys::IO.open("./tmp/"+sp_file+"_ext_time.nc", "time")
    unless time.val[0] == ts && time.val[-1] == te
      time.close
      system('rm -f ./tmp/'+sp_file+'_ext_time.nc')
    end
  end
  unless File.exist?("./tmp/"+sp_file+"_ext_time.nc") 
    vircut = vir.cut("time"=>ts..te)
    time = NArray.sfloat(vircut.length)

    for i in 0..vircut.coord("time").length-1
      time[i] = vircut.coord("time").val[i]
    end

    sagyoufile = NetCDF.create("./tmp/"+sp_file+"_ext_time.nc")
    #sp_file_ext_time.nc の軸定義
    var.axnames.each do |dimname|
      unless dimname == 'time'
        dim = var.coord(dimname)
        sagyoufile.def_dim(dim.name, dim.length)
      end
    end
    sagyoufile.def_dim('time', vircut.length)
    # 時間軸以外の定義
    var.axnames.each do |dimname|
      unless dimname == 'time'
        sagyoufile.redef
        dim = var.coord(dimname)
        sagyou = sagyoufile.def_var(dim.name, dim.ntype, [dim.name])
        for i in 0..(dim.att_names.size - 1)
          sagyou.put_att(dim.att_names[i], dim.get_att(dim.att_names[i]))
        end
        sagyoufile.enddef
        sagyou.put(dim.val)
      end
    end
    sagyoufile.redef
    time_dim = sagyoufile.def_var(vir.coord(-1).name, vir.coord(-1).ntype, [vir.coord(-1).name])
    for i in 0..(vir.coord(-1).att_names.size - 1)
      time_dim.put_att(vir.coord(-1).att_names[i], vir.coord(-1).get_att(vir.coord(-1).att_names[i]))
    end
    timecount = vir.coord(-1).val
    var_axnames = var.axnames << vir.coord(-1).name
    sagyou = sagyoufile.def_var(var.name, var.ntype, var_axnames)
    for i in 0..(var.att_names.size - 1)
      sagyou.put_att(var.att_names[i], var.get_att(var.att_names[i]))
    end
    sagyoufile.enddef
    index=0
    time = sagyoufile.var(vir.axnames[-1])
    axnames = var.shape.size
    array1 = Array.new(axnames, 0)
    array2 = Array.new(axnames,0)
    timecount.each do |times|
      array1.fill(0,0..-1).push(index)
      array2.fill(-1,0..-1).push(index)

      sagyou.put(var.val, "start"=>array1, "end"=>array2)
      time.put(times, "index"=>[index])
      array1.delete_at(-1)
      array2.delete_at(-1)
      index = index + 1
    end
    sagyoufile.close
  end
end

def local_time_to_lon(local_time, unit = 'hour')
  # 経度 0 度が 0 時の時の引数 local_time (ローカルタイムの時間) に合わせた経度を返す. つまりローカルタイムの時間の太陽直下点の経度を与える.
  # local_time が時間なのか秒なのかは unit で考える. lon の場合直接経度を与える. 
  if unit == 'hour'
    lon = local_time * (360.0 / DataNumOfDay)
  elsif unit == 'sec'
    lon = local_time * (360.0 / DataNumOfDay) * HoursOfSecond
  elsif unit == 'lon'
    lon = local_time
  end
  print("lon=",lon,", ")
  if unit == 'hour'
    hour = local_time
  elsif unit == 'sec'
    hour = local_time / HoursOfSecond
  elsif unit == 'lon'
    hour = local_time *(DataNumOfDay / 360.0)
  end
  print("local_time=", hour, "[hour]\n") 
  return lon
end

def lon_to_index(lon, gphys)
  # gphys データで与えられた lon が lon 軸の何番目の要素かを返す

  index_judgment = gphys.cut_rank_conserving("lon"=>lon).coord("lon").val[0]
  index = nil
  gphys.coord("lon").to_a.each_with_index do |lon, index_of_gphys|
    index = index_of_gphys if lon == index_judgment
  end
  return index
end
def remake_dir(figdir,opt)
  #figdirがある時は削除するかを聞く. オプション-yをつけると宣告なしで削除する
  opt.on('-y') {|option|  #オプション設定
    if option then
      if FileTest::directory?(figdir)
        system( 'rm -rf ' + figdir)

        ##$    if File.exists?(figdir) then
        ##$      figfile.each{ |file|
        ##$        next if /^\.+$/ =~ file
        ##$        File.delete(file)}
        ##$      Dir.rmdir(figdir)
      end
    end
  }
  opt.parse!(ARGV)

  if File.exists?(figdir) then
    p "already figs existed. Do you want to remove it ? y or n"
    figrm = gets.chop #キーボード入力の認識(gets) chop は改行を認識しない
    if figrm == "y" then
      system( 'rm -rf ' + figdir)
      ##$    figfile.each{ |file|
      ##$      next if /^\.+$/ =~ file
      ##$      File.delete(file)}
      ##$    Dir.rmdir(figdir)
      ##$    Dir::mkdir( figdir )
    else
      p "you choose n. stop script."
      exit(3)
    end 
  end
  Dir::mkdir( figdir )
end

def laglange_interpolating(x, gphys, hour)
  # laglange_interpolating_core を用いて hour の時間での x (経度) での gphys を求める
  lon_num = gphys.coord("lon").length
  if gphys.cut_rank_conserving("time"=>hour,"lon"=>x).coord("lon").val[0] == x
    y = gphys.cut("time"=>hour,"lon"=>x)
    return y
  else
    array_x = []
    arrray_y = []
    for i in 0..(lon_num - 1)
      array_x << gphys.coord(0).val[i]
      array_y << gphys[i,false].cut("time"=>hour).copy
    end

    y = laglange_interpolating_core(x, array_x, array_y)
    return y
  end
end
#def laglange_interpolating(x, gphys, hour)
#  lon_num = gphys.coord("lon").length
#  if gphys.cut_rank_conserving("time"=>hour,"lon"=>x).coord("lon").val[0] == x
#    y = gphys.cut("time"=>hour,"lon"=>x)
#    return y
#  else
#    y = 0.0
#    for i in 0..(lon_num - 1)
#      unless i == 0
#        l_i[i] = (x - gphys[0,false].cut("time"=>hour).coord(0).val[0])/(gphys[i,false].cut("time"=>hour).coord(0).val[0] - gphys[0,false].cut("time"=>hour).coord(0).val[0])
#      else
#        l_i[i] = (x - gphys[0,false].cut("time"=>hour).coord(0).val[0])/(gphys[i,false].cut("time"=>hour).coord(0).val[0] - gphys[1,false].cut("time"=>hour).coord(0).val[0])
#        for j in 2..(lon_num - 1)
#          l_i[i] = l_i[i] * (x - gphys[j,false].cut("time"=>hour).coord(0).val[0])/(gphys[i,false].cut("time"=>hour).coord(0).val[0] - gphys[j,false].cut("time"=>hour).coord(0).val[0])
#        end
#      end
#      for j in 1..(lon_num - 1)
#        unless i == j
#          l_i[i] = l_i[i] * (x - gphys[j,false].cut("time"=>hour).coord(0).val[0])/(gphys[i,false].cut("time"=>hour).coord(0).val[0] - gphys[j,false].cut("time"=>hour).coord(0).val[0])
#        end
#      end
#      y = y + l_i[i] * gphys[i,false].cut("time"=>hour).copy
#    end
#    return y
#  end
#end
def laglange_interpolating_core(x, x_k, y)
  # ラグランジュ補間を行う. x は補間したい横軸の点, x_k は既知の横軸の点(配列), y は既知の縦軸の点(配列). x の点での 縦軸の値を返す
  var = 0.0
  for i in 0..(lon_num - 1)
    unless i == 0
      l_i[i] = (x - x_k[0]) / (x_k[i] - x_k[0])
    else
      l_i[i] = (x - x_k[0]) / (x_k[i] - x_k[1])
      for j in 2..(lon_num - 1)
        l_i[i] = l_i[i] * (x - x_k[j]) / (x_k[i] - x_k[j])
      end
    end
    for j in 1..(lon_num - 1)
      unless i == j
        l_i[i] = l_i[i] * (x - x_k[j]) / (x_k[i] - x_k[j])
      end
    end
    var = var + l_i[i] * y[i]
  end
  return var
end
def vfratio(fx,fy, factor = 2.0)
  # dclext.rb の flow_vect を参考にベクトルのビューポートに対する格子間隔と強さの比を出す. これを@@vxfxratio, あるいは @@vyfyratio に与えることで GGraph.vect の縦横比をそろえることができる.

  # 引数に与えたものが正しくない場合はエラーをはいて終了
  raise ArgumentError,"Expect 2D arrays" if fx.rank != 2 || fy.rank != 2
  raise ArgumentError,"fx.shape != fy.shape" if fx.shape != fy.shape

  # fx の緯度, 経度の格子点数を取得
  nx, ny = fx.shape  

  # 設定されているビューポート(ビューポートとは表示するウィンドウの縦と横をそれぞれ1 の長さとしたとき, 左端(vx0)を横の0.0右端(vx1)を1.0, 下端(vy0)を縦の0.0, 右端(vy1)を縦の1.0 としたもの) を取得
  vx0,vx1,vy0,vy1 = DCL.sgqvpt

  # 設定されている表示する座標の範囲を取得 GGraph.set_fig 'window'=>[0,360,-90,90] ならば ux0 = 0, ux1 = 360 まで,uy0 =  -90 uy1 =  90
  wnd = DCL.sgqwnd
  if wnd.include?(DCL.glrget('rundef'))
    ux0,ux1,uy0,uy1 = DCL.sgqtxy
  else
    ux0,ux1,uy0,uy1 = wnd
  end
  dvx = (vx1-vx0)/nx         # fx の経度方向の格子点間の距離に対する ビューポートの横の長さ. つまり, 格子点間の距離を 1 としたときのビューポートでの横の長さを表している
  dvy = (vy1-vy0)/ny        # fx の緯度方向の格子点間の距離に対する ビューポートの縦の長さ. つまり, 格子点間の距離を 1 としたときのビューポートでの縦の長さ表している
  ax = (vx1-vx0)/(ux1-ux0)   # ウィンドウに表示した横方向の座標値に対する ビューポートの長さ. つまり, 実際の格子点の長さの範囲を 1 としたときのビューポートの横の長さを表している.
  ay = (vy1-vy0)/(uy1-uy0)   # ウィンドウに表示した縦方向の座標値に対する ビューポートの長さ. つまり, 実際の格子点の長さの範囲を 1 としたときのビューポートの縦の長さを表している.
  fxmx = fx.abs.max.to_f
  fymx = fy.abs.max.to_f
  # fx, fy が不適切ならエラー
  raise "fx has no data or all zero" if fxmx == 0
  raise "fy has no data or all zero" if fymx == 0

  # fx, fy の最大値に対する格子点間距離を比べて小さいを方を選択
  # dvx/(ax*fxmx) = ((vx1-vx0)/(nx))/((vx1-vx0)/(ux1-ux0) * fxmx)
  #               = (ux1 - ux0)/(nx * fxmx)
  # (ux1 - ux0)/nx は fx の経度方向の格子点間距離である.
  # つまり fx の強さの最大値を 1 としたときの格子点間距離を表す
  # それが小さいを選択することで(後で)弱い方の長さを単位として強い方がより長くなる様にしている
  cn = [(dvx/(ax*fxmx)).abs, (dvy/(ay*fymx)).abs].min  # normalztn const
  if (dvx/(ax*fxmx)).abs <= (dvy/(ay*fymx)).abs then
    switch = 'fx'
  else
    switch = 'fy'
  end

  # ax を掛けることで実際のビューポートの長さに焼き直す.
  vxfxratio = factor*cn*ax
  vyfyratio = factor*cn*ay

  return vxfxratio, vyfyratio, switch, vx1 - vx0, vy1 - vy0
end
